@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* test.h  unit test routines
  *
  * Copyright (C) 2003 Red Hat, Inc.
@@ -36,6 +36,7 @@ dbus_bool_t bus_dispatch_test         (const DBusString             *test_data_d
 dbus_bool_t bus_dispatch_sha1_test    (const DBusString             *test_data_dir);
 dbus_bool_t bus_policy_test           (const DBusString             *test_data_dir);
 dbus_bool_t bus_config_parser_test    (const DBusString             *test_data_dir);
+dbus_bool_t bus_config_parser_trivial_test (const DBusString        *test_data_dir);
 dbus_bool_t bus_signals_test          (const DBusString             *test_data_dir);
 dbus_bool_t bus_expire_list_test      (const DBusString             *test_data_dir);
 dbus_bool_t bus_activation_service_reload_test (const DBusString    *test_data_dir);
