@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-marshal-recursive.c  Marshalling routines for recursive types
  *
  * Copyright (C) 2004, 2005 Red Hat, Inc.
@@ -690,6 +690,7 @@ static const DBusTypeReaderClass variant_reader_class = {
   base_reader_next
 };
 
+#ifndef DBUS_DISABLE_ASSERT
 static const DBusTypeReaderClass * const
 all_reader_classes[] = {
   &body_reader_class,
@@ -702,6 +703,7 @@ all_reader_classes[] = {
   &array_types_only_reader_class,
   &variant_reader_class
 };
+#endif
 
 /**
  * Initializes a type reader.
@@ -1652,7 +1654,7 @@ writer_recurse_init_and_check (DBusTypeWriter *writer,
                                      _dbus_type_to_string (expected),
                                      _dbus_string_get_const_data (writer->type_str), writer->type_pos);
           else
-            _dbus_warn_check_failed ("Writing an element of type %s, but no value is expected here\n",
+            _dbus_warn_check_failed ("Writing an element of type %s, but no value is expected here\n"
                                      "The overall signature expected here was '%s' and we are on byte %d of that signature.\n",
                                      _dbus_type_to_string (sub->container_type),
                                      _dbus_string_get_const_data (writer->type_str), writer->type_pos);
