@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-marshal-validate.c Validation routines for marshaled data
  *
  * Copyright (C) 2005 Red Hat, Inc.
@@ -247,6 +247,7 @@ _dbus_validate_signature_with_reason (const DBusString *type_str,
         }
 
       if (last == DBUS_DICT_ENTRY_BEGIN_CHAR &&
+          _dbus_type_is_valid (*p) &&
           !dbus_type_is_basic (*p))
         {
           result = DBUS_INVALID_DICT_KEY_MUST_BE_BASIC_TYPE;
@@ -693,7 +694,7 @@ _dbus_validate_body_with_reason (const DBusString *expected_signature,
 
 /**
  * Checks that the given range of the string is a valid object path
- * name in the D-BUS protocol. Part of the validation ensures that
+ * name in the D-Bus protocol. Part of the validation ensures that
  * the object path contains only ASCII.
  *
  * @todo this is inconsistent with most of DBusString in that
@@ -761,7 +762,7 @@ _dbus_validate_path (const DBusString  *str,
 
 /**
  * Checks that the given range of the string is a valid interface name
- * in the D-BUS protocol. This includes a length restriction and an
+ * in the D-Bus protocol. This includes a length restriction and an
  * ASCII subset, see the specification.
  *
  * @todo this is inconsistent with most of DBusString in that
@@ -837,7 +838,7 @@ _dbus_validate_interface (const DBusString  *str,
 
 /**
  * Checks that the given range of the string is a valid member name
- * in the D-BUS protocol. This includes a length restriction, etc.,
+ * in the D-Bus protocol. This includes a length restriction, etc.,
  * see the specification.
  *
  * @todo this is inconsistent with most of DBusString in that
@@ -898,7 +899,7 @@ _dbus_validate_member (const DBusString  *str,
 
 /**
  * Checks that the given range of the string is a valid error name
- * in the D-BUS protocol. This includes a length restriction, etc.,
+ * in the D-Bus protocol. This includes a length restriction, etc.,
  * see the specification.
  *
  * @todo this is inconsistent with most of DBusString in that
@@ -939,7 +940,7 @@ _dbus_validate_error_name (const DBusString  *str,
 
 /**
  * Checks that the given range of the string is a valid bus name in
- * the D-BUS protocol. This includes a length restriction, etc., see
+ * the D-Bus protocol. This includes a length restriction, etc., see
  * the specification.
  *
  * @todo this is inconsistent with most of DBusString in that
@@ -1039,7 +1040,7 @@ _dbus_validate_bus_name (const DBusString  *str,
 
 /**
  * Checks that the given range of the string is a valid message type
- * signature in the D-BUS protocol.
+ * signature in the D-Bus protocol.
  *
  * @todo this is inconsistent with most of DBusString in that
  * it allows a start,len range that extends past the string end.
@@ -1065,17 +1066,17 @@ _dbus_validate_signature (const DBusString  *str,
 }
 
 /** define _dbus_check_is_valid_path() */
-DEFINE_DBUS_NAME_CHECK(path);
+DEFINE_DBUS_NAME_CHECK(path)
 /** define _dbus_check_is_valid_interface() */
-DEFINE_DBUS_NAME_CHECK(interface);
+DEFINE_DBUS_NAME_CHECK(interface)
 /** define _dbus_check_is_valid_member() */
-DEFINE_DBUS_NAME_CHECK(member);
+DEFINE_DBUS_NAME_CHECK(member)
 /** define _dbus_check_is_valid_error_name() */
-DEFINE_DBUS_NAME_CHECK(error_name);
+DEFINE_DBUS_NAME_CHECK(error_name)
 /** define _dbus_check_is_valid_bus_name() */
-DEFINE_DBUS_NAME_CHECK(bus_name);
+DEFINE_DBUS_NAME_CHECK(bus_name)
 /** define _dbus_check_is_valid_signature() */
-DEFINE_DBUS_NAME_CHECK(signature);
+DEFINE_DBUS_NAME_CHECK(signature)
 
 /** @} */
 
